/*
	GBA

	Variables d'environnement
	-------------------------
	Fonctions de lecture: doivent remplir *cmode avec la bonne valeur!
	Grer les autres pixelformats (crecouleurs)!
	Chaque entre de la palette: 3 octets
		- Bleu
		- Vert
		- Rouge
	(!!!) Palette 32 bits (RGBA)!
*/

namespace pluginGBA
{
	int fctInitConversionGBA(int flags, void *data);
	int fctLitPaletteGBA(int flags, int *cmode, unsigned char *palette, char *nom_fichier);
	int fctLitTilesetGBA(int flags, int *cmode, unsigned char *palette,unsigned long **motiv, const char *nom, int original);
	int fctLitMapGBA(int flags, int *cmode, unsigned char *palette, unsigned long **motifs, MAP **umap, int i);
	int fctLitListeGBA(int flags, int *cmode, unsigned char *palette, unsigned long **motifs, MAP **umap, int i);
	int fctLitBitmapGBA(int flags, int *cmode, unsigned char *palette,unsigned long **motiv, const char *nom, int original);
	int fctEcritPaletteGBA(int flags, int *cmode, ELEMENT *e, unsigned char *palette, int nbCouleurs);
	int fctEcritTilesetGBA(int flags, int *cmode, ELEMENT *e, ELEMENT *palfile, unsigned long *motifs, unsigned char *palette, int nTiles, int tileX, int tileY);
	int fctEcritMapGBA(int flags, int *cmode, ELEMENT *e, ELEMENT *tilfile, ELEMENT *palfile, unsigned long *map, int tailleX, int tailleY, int tileX, int tileY);
	int fctEcritListeGBA(int flags, int *cmode, ELEMENT *e, ELEMENT *tilfile, ELEMENT *palfile, SPRITE_LISTE *sl, char *typeSd, void *userData, SD_PARAMS *modele, int tileX, int tileY);
	int fctEcritBitmapGBA(int flags, int *cmode, ELEMENT *e, ELEMENT *palfile, unsigned long *tableau, unsigned char *palette, int tailleX, int tailleY);
	int fctPersonnalisationGBA(int flags, int *cmode);
	int bGBAMirrorMask, bGBAPaletteMask, nGBAAlignement, bGBAColorOrder;
	PLATFORM_INFO *ptrPFGBA;

	PF_PIXELFORMAT pixelGBA=	{
		2,						//16 bits (pas utilis pour le moment)
		5, 5, 5, 0,				//RGB(A pas gr pour le moment)
	};
	PF_PIXELFORMAT pixelLossless=	{
		4,						//32 bits
		8, 8, 8, 8,				//RGBA
	};

	PF_PALETTEFORMAT pixelFormatListGBA[]=		{
		{
			//4 bits palette
			"4 bits (16 coul.)",
			"4", 4,						//Identificateur
			2,							//Conversion optimise 16x16 palettes (hack)
			TRUE,						//Color keying
			1,							//Palettes arrondies  16
			255, 16, 16,				//Max. 16 couleurs, 16 couleurs par palette
			TRUE,
			NULL,
			&pixelGBA
		},
		{
			//8 bits palette
			"8 bits (256 coul.)",
			"8", 8,
			TRUE,
			TRUE,						//Color keying
			FALSE,
			256, 0,	256,				//256 couleurs
			TRUE,
			NULL,
			&pixelGBA
		},
		{
			//15 bits
			"15 bits (32k coul.)",
			"15", 15,
			FALSE,
			TRUE,						//Color keying
			FALSE,
			0, 0, 0,
			FALSE,
			NULL,
			&pixelGBA
		},
/*		{
			//24 bits avec color keying (RGB only)
			"24 bits (RGB)",
			"24", 24,
			FALSE,
			TRUE,						//Color keying
			FALSE,
			0, 0, 0,
			FALSE,
			NULL,
			&pixelLossless
		},*/
		{
			//32 bits sans color keying (RGBA)
			"32 bits (RGBA)",
			"32", 32,
			FALSE,
			FALSE,						//Color keying
			FALSE,
			0, 0, 0,
			FALSE,
			NULL,
			&pixelLossless
		}
	};

	int Rvba15Bits(int r, int g, int b, int a) {
		return bGBAColorOrder ?
			((b)|((g)<<5)|((r)<<10)|((a)<<15)) :
			((r)|((g)<<5)|((b)<<10)|((a)<<15));
	}
	void LitCouleur(int c, unsigned char *palette) {
		if (bGBAColorOrder) {
			*palette++=(c&0x1F);
			*palette++=((c&0x3E0)>>5);
			*palette++=((c&0x7C00)>>10);
			*palette++=(c&0x8000)?0:255;
		}
		else {
			*palette++=((c&0x7C00)>>10);
			*palette++=((c&0x3E0)>>5);
			*palette++=(c&0x1F);
			*palette++=(c&0x8000)?0:255;
		}
	}
	int LitCouleurRGBA(int c) {
		unsigned char pal[4];
		LitCouleur(c, pal);
		return RGBA(pal[0], pal[1], pal[2], pal[3]);
	}

	void FctGBAApplyAlign(int value)			{
		//%user[2]%
		if (value)
			defUserData[1] = "__attribute__((aligned(4))) ";
		else
			defUserData[1] = "";
	}

	int fctInitConversionGBA(int flags, void *data)
	{
		FctGBAApplyAlign(nGBAAlignement);
		return 1;
	}

	int fctLitTilesetGBA(int flags, int *cmode, unsigned char *palette,unsigned long **motiv, ELEMENT *e, int original)		{
		FILE *f;
		int x,c,k,j,z, nEcrits, realTileX, realTileY;
		unsigned long *motifs;
		char bits[100];
		char *bloc=(char*)malloc(TAILLE_MAXIMUM);
		int taille_totale;

		motifs=*motiv;

		ChoisitFormatFichierExtension(e->nom_fichier);
		f=fopen(e->nom_fichier,"rb");
		if (!f)		{
			free(bloc);
			MessageBox(gblHDlg,"Le fichier tileset spcifi n'existe pas",e->nom_fichier,MB_ICONERROR);
			return 0;
		}

		ReinitSymTable();
		PrepareSymTable(1);

		InitSymbole(6,gblcPal);							//%link%
		InitSymbole(12,bits);							//%bits%
		bits[0]=0;

		taille_totale=fread(bloc,1,TAILLE_MAXIMUM,f);

		//Lit ventuellement ce dont on a besoin dans le ref file
		refFile::readSectionFiltree(til_refdefine, e);

		k=LitChaineFiltree(0,bloc,til_declaration, taille_totale);
		if (k>taille_totale)
			return 0;

		taille=(int)symboles[3].donnees;

		if ((int)symboles[9].donnees==32)						//%depth%
			*cmode=3;
		else if ((int)symboles[9].donnees==15)					//%depth%
			*cmode=2;
		else if ((int)symboles[9].donnees==4)					//%depth%
			*cmode=0;
		else
			*cmode=1;
		//Pour trouver la profondeur (bits)
		if (palette && curPF->pfList[*cmode].bUsePalettes)			{
			ELEMENT e = makeElement(gblcPal, TYPE_PALETTE);
			if (!LitPalette(palette, &e, FALSE, FALSE))		{
				free(bloc);
				fclose(f);
				return 0;
			}
		}

		//Toujours du 8x8!!!!
		taille*=(tileX/8)*(tileY/8);

		vraieTaille=taille;

		if (!motifs)		{
			motifs=(unsigned long*)malloc(tileX*tileY*taille*sizeof(long));
			utilMemoire+=tileX*tileY*taille*sizeof(long);
			*motiv=motifs;
		}


		nEcrits=0;

		realTileX=tileX, realTileY=tileY;
		tileX=tileY=8;
		for (j=0;j<taille;j++)		{
			if (j>=TILEMAX)		{
				MessageBox(gblHDlg,"Le nombre de tiles est suprieur  la mmoire. Le format serait-il faux?",e->nom_fichier,MB_ICONERROR);
				fclose(f);
				free(motifs);
				*motiv=NULL;
				taille=0;
				free(bloc);
				return taille-1;
			}

			for (x=0;x<tileY*tileX;x++)			{
				if (*til_fin_ligne)
					z=x;
				else
					z=nEcrits;

				if (til_nbChaqueX>0 && z>0 && z%til_nbChaqueX==0)
					k=LitChaineFiltree(k,bloc,til_chaque_x, taille_totale);

				if (curPF->pfList[*cmode].ID == 32)			//32 bits RGBA
					k = LitValeur(k,bloc,til_chaque_valeur_3,f,&c,taille_totale);
				else if (curPF->pfList[*cmode].ID == 15)	//15 bits
					k=LitValeur(k,bloc,til_chaque_valeur_2,f,&c,taille_totale);
				else
					k=LitValeur(k,bloc,til_chaque_valeur_1,f,&c,taille_totale);

				if (curPF->pfList[*cmode].ID == 15)		{
					if (!original)
						c = LitCouleurRGBA(c);
					// FIXME tendre la couleur  32 bits si !original
					motifs[(j*tileX*tileY)+x]=c;
				}
				else if (curPF->pfList[*cmode].ID == 4)		{
					motifs[(j*tileX*tileY)+x+1]=(c>>4)&0xf;
					motifs[(j*tileX*tileY)+x]=c&0xf;
					x++;
				}
				else
					motifs[(j*tileX*tileY)+x]=c;

//				LitValeur(k,bloc,til_chaque_valeur_2,f,&c);
				nEcrits++;
			}
			k=LitChaineFiltree(k,bloc,til_fin_ligne, taille_totale);
		}
		tileX=realTileX, tileY=realTileY;

		if (vraieTaille<taille)
			taille=vraieTaille;

		fclose(f);
		free(bloc);
		return taille;
	}

	int fctLitPaletteGBA(int flags, int *cmode, unsigned char *palette, ELEMENT *e)		{
		FILE *f;
		int c,j, nbCouleurs;
		char *bloc=(char*)malloc(TAILLE_MAXIMUM);
		u8 *palBase = palette;
		int taille_totale;

		ChoisitFormatFichierExtension(e->nom_fichier);
		f=fopen(e->nom_fichier,"rb");
		if (!f)		{
			free(bloc);
			MessageBox(gblHDlg,"Le fichier palette spcifi n'existe pas.", e->nom_fichier, MB_ICONERROR);
			return 0;
		}

		ReinitSymTable();
		PrepareSymTable(1);

		InitSymbole(6,gblcPal);							//%link%

		taille_totale=fread(bloc,1,TAILLE_MAXIMUM,f);

		//Lit ventuellement ce dont on a besoin dans le ref file
		refFile::readSectionFiltree(pal_refdefine, e);

		j=LitChaineFiltree(0,bloc,pal_declaration, taille_totale);
		if (j>taille_totale)
			return 0;


		nbCouleurs=0;
	//	palette+=3;
		memset(palette,0,256*4);

		while(1)		{
			j=LitValeur(j,bloc,pal_chaque_valeur_1,f,&c, taille_totale);
			if (j>taille_totale)
				break;

			LitCouleur(c, palette);
			palette += 4;
			nbCouleurs++;

			if (pal_nbChaqueX>0 && nbCouleurs>0 && nbCouleurs%pal_nbChaqueX==0)
				j=LitChaineFiltree(j,bloc,pal_chaque_x, taille_totale);

			if (nbCouleurs>=256)
				break;
		}

		//Premire couleur toujours transparente
		if (ptrPFGBA->pfList[curCMode].bFirstColorTransparent)
			*(u32*)palBase = 0;

		fclose(f);
		free(bloc);

		//Toujours le mme type 1
		*cmode = curPF->pfDefaut;
		return nbCouleurs;
	}

	int fctLitMapGBA(int flags, int *cmode, unsigned char *palette, unsigned long **motifs, MAP **umap, ELEMENT *e)		{
		FILE *f;
		char temp[4096];
		int x,y,c, ii, j, tX, tY, z;
		MAP *map=*umap;
		char *bloc=(char*)malloc(TAILLE_MAXIMUM);
		char tempGblUserData[NB_MAX_USERDATA][TAILLE_MAX_CHAINES];
		int taille_totale;

		strcpy(gblcMap,e->nom_fichier);

		ChoisitFormatFichierExtension(e->nom_fichier);
		f=fopen(e->nom_fichier,"rb");
		if (!f)		{
			free(bloc);
			MessageBox(gblHDlg,"Le fichier map spcifi n'existe pas",e->nom_fichier,MB_ICONERROR);
			return 0;
		}

		ReinitSymTable();
		PrepareSymTable(1);
		InitSymbole(5,gblcNomC);						//%name%
		InitSymbole(6,gblcTil);							//%link%
		InitSymbole(10,temp);							//%type%
		gfxmask=0xffffffff;

		taille_totale=fread(bloc,1,TAILLE_MAXIMUM,f);
		ii=taille_totale;

		//Lit ventuellement ce dont on a besoin dans le ref file
		refFile::readSectionFiltree(map_refdefine, e);

		j=LitChaineFiltree(0,bloc,map_declaration, taille_totale);
		if (j>taille_totale)
			return 0;
		memcpy(tempGblUserData, gblUserData, sizeof(gblUserData));

		tX=tileX, tY=tileY;

	//	RedimDIB(tailleX*tileX,tailleY*tileY);

	//	InitVariables();					//Rinitialise le systme de variables...
	//	memset(chneVariables,0,sizeof(chneVariables));

		if ((int)symboles[9].donnees==0)	{					//%depth%
			if (!strcmp(temp,type32))							//%bits%
				ModeTexte=2;
			else if (!strcmp(temp,type8))							//%bits%
				ModeTexte=0;
			else
				ModeTexte=1;
		}
		else if ((int)symboles[9].donnees==32)					//%depth%
			ModeTexte=2;
		else if ((int)symboles[9].donnees==8)					//%depth%
			ModeTexte=0;
		else
			ModeTexte=1;


		if (*gblcTil)		{
			ELEMENT e = makeElement(gblcTil, TYPE_MOSAIQUE);
			taille=LitTileset(palette, motifs, &e, 0);
		}
		if (!taille)		{
			free(bloc);
			fclose(f);
			return 0;
		}

		if (!(*umap))		{
			map=(MAP*)malloc(tailleX*tailleY*sizeof(MAP));
			//Dcommenter pour obtenir la consommation mmoire dans l'diteur (il y en a ailleurs  dcommenter encore)
	//		utilMemoire+=tailleX*tailleY*2;
			memset(map,0,tailleX*tailleY*sizeof(MAP));
			*umap=map;
		}

		taille_totale=ii;

		tileX=tX, tileY=tY;

		taille/=(tileX*tileY)/64;

		for (y=0;y<tailleY;y++)			{
			for (x=0;x<tailleX;x++)		{
				if (*map_fin_ligne)
					z=x;
				else
					z=(y*tailleX)+x;

				if (map_nbChaqueX>0 && z>0 && z%map_nbChaqueX==0)
					j=LitChaineFiltree(j,bloc,map_chaque_x, taille_totale);

				if (ModeTexte==2)
					j=LitValeur(j,bloc,map_chaque_valeur_3,f,&c, taille_totale);
				else if (ModeTexte==1)
					j=LitValeur(j,bloc,map_chaque_valeur_1,f,&c, taille_totale);
				else
					j=LitValeur(j,bloc,map_chaque_valeur_2,f,&c, taille_totale);

				if (j>taille_totale)
					goto fin;

				map[tailleX*y+x]=c;
			}
			j=LitChaineFiltree(j,bloc,map_fin_ligne, taille_totale);
		}

	fin:
		fclose(f);
		free(bloc);
		memcpy(gblUserData, tempGblUserData, sizeof(gblUserData));

		return 1;
	}

	int fctLitListeGBA(int flags, int *cmode, unsigned char *palette, unsigned long **motifs, MAP **umap, ELEMENT *e)		{
		FILE *f;
		char temp[4096];
		int i, ii, j, tX, tY, lastJ;
		MAP *map=*umap;
		char *bloc=(char*)malloc(TAILLE_MAXIMUM);
		char nomModele[TAILLE_MAX_CHAINES];
		char tempGblUserData[NB_MAX_USERDATA][TAILLE_MAX_CHAINES];
		int taille_totale;

		strcpy(gblcMap,e->nom_fichier);

		ChoisitFormatFichierExtension(e->nom_fichier);
		f=fopen(e->nom_fichier,"rb");
		if (!f)		{
			free(bloc);
			MessageBox(gblHDlg,"Le fichier map spcifi n'existe pas",e->nom_fichier,MB_ICONERROR);
			return 0;
		}

		strcpy(nomModele, sdParams[gblSdModele].sdNomModele);
		ReinitSymTable();
		PrepareSymTable(1);
		InitSymbole(5,gblcNomC);						//%name%
		InitSymbole(6,gblcTil);							//%link%
		InitSymbole(10,gblTypeSd);						//%type%
		InitSymbole(15,nomModele);						//%model%
		gfxmask=0xffffffff;

		taille_totale=fread(bloc,1,TAILLE_MAXIMUM,f);
		ii=taille_totale;

		//Lit ventuellement ce dont on a besoin dans le ref file
		refFile::readSectionFiltree(lst_refdefine, e);

		j=LitChaineFiltree(0,bloc,lst_declaration, taille_totale);
		if (j>taille_totale)
			return 0;

		memcpy(tempGblUserData, gblUserData, sizeof(gblUserData));
		if (!AppliqueSdModele(nomModele))			{
			sprintf(str, "Impossible de spcifier le modle de paramtres spcifi: %s", nomModele);
			MessageBox(gblHDlg, str, "Erreur", MB_ICONEXCLAMATION);
			return 0;
		}

		MajFichierParams();

		//Initialisation de la liste
		gblSpriteListe.sizeofSD=sdParams[gblSdModele].sizeofSD;
		gblSpriteListe.sd=(SPRITE_DEF*)realloc(gblSpriteListe.sd, NB_MAX_ELE_LISTE*gblSpriteListe.sizeofSD);
		gblSpriteListe.nbSprites=(int)symboles[3].donnees;

		tX=tileX, tY=tileY;

		ELEMENT el = makeElement(gblcTil, TYPE_MOSAIQUE);
		taille=LitTileset(palette, motifs, &el, 0);
		if (!taille)		{
			free(bloc);
			fclose(f);
			return 0;
		}

		taille_totale=ii;
		tileX=tX, tileY=tY;
		tailleX/=tileX; tailleY/=tileY;
		taille/=(tileX*tileY)/64;

		if (gblSpriteListe.nbSprites>NB_MAX_ELE_LISTE)			{
			free(bloc);
			fclose(f);
			MessageBox(NULL, "Impossible de charger la liste car elle contient plus d'lments qu'allous. Modifiez le nombre maximal d'lments par liste dans les options.", e->nom_fichier, MB_ICONEXCLAMATION);
			return 0;
		}

		gblErrorFlag = 0;
		lastJ = j;
		for (i=0;i<gblSpriteListe.nbSprites;i++)		{
			if (lst_nbChaqueX>0)		{
				if (i%lst_nbChaqueX==0)
					j=LitChaineFiltree(j,bloc,lst_chaque_x, taille_totale);
			}

			if (lst_chaque_valeur_2[0])
			{
				/* Nouveau systme, plus souple */
				gblCurrentSprite = _SD(gblSpriteListe, i);
				j = LitChaineFiltree(j, bloc, lst_chaque_valeur_2, taille_totale);
				gblCurrentSprite = NULL;
			}
			else
				/* Ancien systme */
				j=LitChaineTypes(j,bloc,lst_chaque_valeur_ok,f,_SD(gblSpriteListe,i), 0, NULL, taille_totale);

			if (j>taille_totale)
				break;

			j=LitChaineFiltree(j,bloc,lst_fin_ligne, taille_totale);
		}

		/* Ce flag d'erreur n'est l que pour la compatibilit, normalement il faut l'viter,
		   d'ailleurs avec le nouveau systme, ce genre de cas, qui se produit lorsque la
		   structure des fichiers liste a t change au milieu d'un projet (ajout d'un
		   paramtre par exemple) n'est plus support  cause du support pour les fichiers
		   binaires. */
		if (gblErrorFlag)		{
			SD_PARAMS *sdp=&sdParams[gblSdModele];
			memset(gblSpriteListe.sd, 0, NB_MAX_ELE_LISTE*gblSpriteListe.sizeofSD);
			if (MessageBox(gblHDlg, "Le format de la liste est incorrect, un algorithme plus souple et dductif va tre tente pour pouvoir rcuprer le fichier malgr d'ventuels changements dans le modle de paramtres. Cette opration peut chouer, vous devriez donc vrifier si la liste d'objets est bien correcte et ne l'enregistrer que si c'est le cas.", "Attention", MB_ICONEXCLAMATION|MB_OKCANCEL)!=IDOK)			{
				fclose(f);
				free(bloc);
				return 0;
			}
			for (j=0;j<gblSpriteListe.nbSprites;j++)		{
				SPRITE_DEF *sd=_SD(gblSpriteListe,j);
				for (i=0;i<sdp->sdNumParams;i++)			{
					if (sdp->sdTypeParam[i]==SDT_ENTIER)			{
						int *ptr;
						ptr= (int*)((char*)sd->params+GetParamOffset(i));
						*ptr = GetEntier(sdp->sdTexteDefaut[i]);
					}
					else if (sdp->sdTypeParam[i]==SDT_BRUT)			{
						char *cptr;
						cptr=(char*)sd->params+GetParamOffset(i);
						strcpy(cptr,sdp->sdTexteDefaut[i]);
					}
				}
			}
	//		memset(gblSpriteListe.sd, 0, NB_MAX_ELE_LISTE*gblSpriteListe.sizeofSD);
			//Erreur -> tente de l'importer  la manire forte!
			gblErrorFlag = 0;
			j = lastJ;
			//Fin de ligne
			if (!lst_fin_ligne[0])		{
				//Depuis le dernier #
				for (i=strlen(lst_chaque_valeur_ok)-1;i>=0;i--)		{
					if (lst_chaque_valeur_ok[i]=='#')
						break;
				}
				if (i>=0)
					strcpy(temp, lst_chaque_valeur_ok+i+1);
			}
			else
				strcpy(temp, lst_fin_ligne);
			for (i=0;i<gblSpriteListe.nbSprites;i++)		{
				if (lst_nbChaqueX>0)		{
					if (i%lst_nbChaqueX==0)
						j=LitChaineFiltree(j,bloc,lst_chaque_x, taille_totale);
				}

				j=LitChaineTypes(j,bloc,lst_chaque_valeur_ok,f,_SD(gblSpriteListe,i), 2, temp, taille_totale);
				if (j>taille_totale)
					break;

				j=LitChaineFiltree(j,bloc,lst_fin_ligne, taille_totale);
			}
		}

		fclose(f);
		free(bloc);
		memcpy(gblUserData, tempGblUserData, sizeof(gblUserData));

		return 1;
	}

	int fctLitBitmapGBA(int flags, int *cmode, unsigned char *palette,unsigned long **motiv, ELEMENT *e, int original)		{
		FILE *f;
		int x,y,c,j,z, nbCouleurs;
		unsigned long *motifs;
		char bits[100];
		char *bloc=(char*)malloc(TAILLE_MAXIMUM);
		int taille_totale;

		motifs=*motiv;

		ChoisitFormatFichierExtension(e->nom_fichier);
		f=fopen(e->nom_fichier,"rb");
		if (!f)		{
			free(bloc);
			MessageBox(gblHDlg, "Le fichier bitmap spcifi n'existe pas", e->nom_fichier, MB_ICONERROR);
			return 0;
		}

		ReinitSymTable();
		PrepareSymTable(1);

		InitSymbole(6,gblcPal);							//%link%
		InitSymbole(12,bits);							//%bits%
		bits[0]=0;
		gblcPal[0]=0;

		taille_totale=fread(bloc,1,TAILLE_MAXIMUM,f);

		//Lit ventuellement ce dont on a besoin dans le ref file
		refFile::readSectionFiltree(bmp_refdefine, e);

		j=LitChaineFiltree(0,bloc,bmp_declaration, taille_totale);
		if (j>taille_totale)
			return 0;

//		taille=(int)symboles[3].donnees;
		//Pour trouver la profondeur (bits)
		if ((int)symboles[9].donnees==32)						//%depth%
			*cmode=3;
		else if ((int)symboles[9].donnees==15)					//%depth%
			*cmode=2;
		else if ((int)symboles[9].donnees==4)					//%depth%
			*cmode=0;
		else
			*cmode=1;

		if (palette && curPF->pfList[*cmode].bUsePalettes && gblcPal[0])			{
			ELEMENT eTemp = makeElement(gblcPal, TYPE_PALETTE);

			nbCouleurs = LitPalette(palette,&eTemp,FALSE, FALSE);
			if (!nbCouleurs)		{
				free(bloc);
				fclose(f);
				return 0;
			}
		}

		if (!motifs)		{
			motifs=(unsigned long*)malloc(tailleX*tailleY*sizeof(long));
			utilMemoire+=tailleX*tailleY*sizeof(long);
			*motiv=motifs;
		}

		for (y=0;y<tailleY;y++)			{
			for (x=0;x<tailleX;x++)		{
				if (*bmp_fin_ligne)
					z=x;
				else
					z=x+y*tailleX;

				if (bmp_nbChaqueX>0 && z>0 && z%bmp_nbChaqueX==0)
					j=LitChaineFiltree(j,bloc,bmp_chaque_x, taille_totale);

				if (curPF->pfList[*cmode].ID == 32)			//32 bits RGBA
					j = LitValeur(j,bloc,bmp_chaque_valeur_3,f,&c, taille_totale);
				else if (curPF->pfList[*cmode].ID == 15)	//15 bits
					j=LitValeur(j,bloc,bmp_chaque_valeur_2,f,&c, taille_totale);
				else
					j=LitValeur(j,bloc,bmp_chaque_valeur_1,f,&c, taille_totale);

				if (!curPF->pfList[*cmode].bUsePalettes)		{
					if (curPF->pfList[*cmode].ID == 15)		{
						if (!original)
							c = LitCouleurRGBA(c);
					}
					motifs[tailleX*y+x]=c;
/*					if (c == 0xffffffff)
						c = 0xff000000 | RGB(GetBValue(cFond), GetGValue(cFond), GetRValue(cFond));
					else
						c= RGB (((c&0x7C00)>>10)<<3, ((c&0x3E0)>>5)<<3, (c&0x1F)<<3);*/
				}
				motifs[tailleX*y+x] = c;
			}
			j=LitChaineFiltree(j,bloc,bmp_fin_ligne, taille_totale);
		}

		fclose(f);
		free(bloc);
		return 1;
	}





	int fctEcritPaletteGBA(int flags, int *cmode, ELEMENT *e, unsigned char *palette, int nbCouleurs)			{
		FILE *ecri;
		int i;

		//Ecrit la palette
		ecri=fopen(e->nom_fichier, "wb");
		if (!ecri)		{
			CacheTravail();
			MessageBox(gblHDlg,"Impossible d'ouvrir le fichier en criture.\nLe nom est probablement faux.",e->nom_fichier,MB_ICONERROR);
			return 0;
		}

		ReinitSymTable();
		if ((flags & 3) != PL_OPEN)
			PrepareSymTable(2);
		VerifNomVar(e->nom_court,chneNomVarPal);
		InitSymbole(0,e->nom_base);									//%file%
		InitSymbole(3,nbCouleurs);									//%size%
		InitSymbole(4,nbCouleurs*2);								//%memory%
		InitSymbole(5,chneNomVarPal);								//%name%

		//Ecrit ventuellement ce dont on a besoin dans le ref file
		refFile::writeSectionFiltree(pal_refdefine, e);

		EcritChaineFiltree(pal_declaration,ecri);
	//	sprintf(str,"//%s\n//Converti avec GBA Graphics par Brnni\n//Palette\n//Taille: %i\n//Mmoire: %i octets\n\nconst unsigned short %s_palette[%i]=	{\n",els[0].nom_base,nbCouleurs,nbCouleurs*2,els[1].nom_court,nbCouleurs);
	//	fputs(str,ecri);
	//	nEcrits=0;

		for (i=0;i<nbCouleurs;i++)		{
			if (pal_nbChaqueX>0 && i>0 && i%pal_nbChaqueX==0)
				EcritChaineFiltree(pal_chaque_x,ecri);
			myprintf(ecri,pal_chaque_valeur_1, Rvba15Bits(palette[4*i+2],palette[4*i+1],palette[4*i],!(palette[4*i+3]&1)));
		}
		EcritChaineFiltree(pal_fin_declaration,ecri);
		fclose(ecri);
		return 1;
	}

	int fctEcritTilesetGBA(int flags, int *cmode, ELEMENT *e, ELEMENT *palfile, unsigned long *motifs, unsigned char *palette, int nTiles, int tileX, int tileY)		{
		int i, j, z, nEcrits;
		FILE *ecri;

		ecri=fopen(e->nom_fichier,"wb");
		if (!ecri)		{
			CacheTravail();
			MessageBox(gblHDlg,"Impossible d'ouvrir le fichier en criture.\nLe nom est probablement faux ou vous n'avez pas les autorisations d'accs dans ce rpertoire (seriez-vous  l'cole?).",e->nom_fichier,MB_ICONERROR);
			return 0;
		}

		i=nTiles*tileX*tileY;
		if (curPF->pfList[*cmode].ID == 4)		i/=2;

		ReinitSymTable();
		if ((flags & 3) != PL_OPEN)
			PrepareSymTable(2);
		VerifNomVar(e->nom_court,chneNomVarTil);
		InitSymbole(0,e->nom_base);								//%file%
	//	InitSymbole(3,nTiles*((tileX*tileY)/(8*8)));				//%size%
		InitSymbole(3,nTiles);										//%size%
		InitSymbole(4,i);											//%memory%
		InitSymbole(5,chneNomVarTil);								//%name%
		if (curPF->pfList[*cmode].bUsePalettes)
			InitSymbole(6,palfile->nom_fichier);					//%link%
		else
			InitSymbole(6,"");										//%link%
		InitSymbole(9,curPF->pfList[*cmode].ID);					//%depth%
		if (curPF->pfList[*cmode].ID == 32)
			InitSymbole(10,type32);									//%type%
		else if (curPF->pfList[*cmode].ID == 15)
			InitSymbole(10,type16);									//%type%
		else
			InitSymbole(10,type8);									//%type%

		//Ecrit ventuellement ce dont on a besoin dans le ref file
		refFile::writeSectionFiltree(til_refdefine, e);
		EcritChaineFiltree(til_declaration,ecri);

	//	sprintf(str,"//%s\n//Converti avec GBA Graphics par Brnni\n//Tileset%s(\"%s\")\n//Taille: %i*%i*%i\n//Mmoire: %i octets\n\nconst unsigned char %s_tiles[%i]=	{\n",els[0].nom_base,b16coul?" 4 bits ":" ",els[1].nom_fichier,8,8,nTiles*(tileX/8)*(tileY/8),i,els[0].nom_court,i);
	//	fputs(str,ecri);

		nEcrits=0;
		//Toujours du 8x8
		nTiles*=(tileX/8)*(tileY/8);
		tileX=tileY=8;

		for (i=0;i<nTiles;i++)		{
			for (j=0;j<tileX*tileY;j++)		{
				if (*til_fin_ligne)
					z=j;
				else
					z=nEcrits;

				if (til_nbChaqueX>0 && z>0 && z%til_nbChaqueX==0)
					EcritChaineFiltree(til_chaque_x,ecri);

				if (curPF->pfList[*cmode].ID == 32)			{
					unsigned char *c = (unsigned char*)&motifs[(i*tileX*tileY)+j];
					myprintf(ecri,til_chaque_valeur_3, RGBA(c[0],c[1],c[2],c[3]));
				}
				else if (curPF->pfList[*cmode].ID == 15)			{
					unsigned char *c = (unsigned char*)&motifs[(i*tileX*tileY)+j];
					myprintf(ecri,til_chaque_valeur_2, Rvba15Bits(c[2],c[1],c[0],!(c[3]&1)));
				}
				else if (curPF->pfList[*cmode].ID == 4)
					myprintf(ecri,til_chaque_valeur_1,((motifs[(i*tileX*tileY)+j+1]&0x0f)<<4)|(motifs[(i*tileX*tileY)+j]&0x0f));
				else
					myprintf(ecri,til_chaque_valeur_1,motifs[(i*tileX*tileY)+j]);

				if (curPF->pfList[*cmode].ID == 4)
					j++;

				nEcrits++;
			}
			EcritChaineFiltree(til_fin_ligne,ecri);
		}
		EcritChaineFiltree(til_fin_declaration,ecri);

		fclose(ecri);
		return 1;
	}


	int fctEcritMapGBA(int flags, int *cmode, ELEMENT *e, ELEMENT *tilfile, ELEMENT *palfile, unsigned long *map, int tailleX, int tailleY, int tileX, int tileY)		{
		FILE *ecri;
		int i, j, nEcrits, z;

		ecri=fopen(e->nom_fichier,"wb");
		if (!ecri)		{
			CacheTravail();
			MessageBox(gblHDlg,"Impossible d'ouvrir le fichier en criture.\nLe nom est probablement faux.",e->nom_fichier,MB_ICONERROR);
			return 0;
		}

	/*	tailleX/=tileX;
		tailleY/=tileY;*/

		ReinitSymTable();
		if ((flags & 3) != PL_OPEN)
			PrepareSymTable(2);
		VerifNomVar(e->nom_court,chneNomVarMap);
		InitSymbole(0,e->nom_base);								//%file%
		InitSymbole(1, &tailleX);								//%sizex%
		InitSymbole(2, &tailleY);								//%sizey%
		i=tailleX*tailleY;
		if (ModeTexte==2)
			InitSymbole(4,tailleX*tailleY*4);						//%memory%
		else if (ModeTexte==1)
			InitSymbole(4,tailleX*tailleY*2);						//%memory%
		else
			InitSymbole(4,tailleX*tailleY);							//%memory%
		InitSymbole(5,chneNomVarMap);								//%name%
		InitSymbole(6,tilfile->nom_fichier);						//%link%
		if (ModeTexte==0)
			InitSymbole(9,8);										//%depth%
		else if (ModeTexte==1)
			InitSymbole(9,16);										//%depth%
		else if (ModeTexte==2)
			InitSymbole(9,32);										//%depth%
		if (ModeTexte==2)
			InitSymbole(10,type32);									//%type%
		else if (ModeTexte)
			InitSymbole(10,type16);									//%type%
		else
			InitSymbole(10,type8);									//%type%

		//Ecrit ventuellement ce dont on a besoin dans le ref file
		refFile::writeSectionFiltree(map_refdefine, e);
		EcritChaineFiltree(map_declaration,ecri);

		nEcrits=0;
		for (j=0;j<tailleY;j++)		{
			for (i=0;i<tailleX;i++)		{
				if (*map_fin_ligne)
					z=i;
				else
					z=(j*tailleX)+i;

				if (map_nbChaqueX>0 && z>0 && z%map_nbChaqueX==0)
					EcritChaineFiltree(map_chaque_x,ecri);

				if (ModeTexte==2)
					myprintf(ecri,map_chaque_valeur_3,map[tailleX*j+i]);
				else if (ModeTexte==1)
					myprintf(ecri,map_chaque_valeur_1,map[tailleX*j+i]);
				else
					myprintf(ecri,map_chaque_valeur_2,map[tailleX*j+i]);

				nEcrits++;
			}
			EcritChaineFiltree(map_fin_ligne,ecri);
		}
		EcritChaineFiltree(map_fin_declaration,ecri);

		fclose(ecri);
		return 1;
	}

	int fctEcritListeGBA(int flags, int *cmode, ELEMENT *e, ELEMENT *tilfile, ELEMENT *palfile, SPRITE_LISTE *sl, char *typeSd, void *userData, SD_PARAMS *modele, int tileX, int tileY)			{
		SPRITE_LISTE listeSauve;
		FILE *ecri;
		int i,j;

		ecri=fopen(e->nom_fichier,"wb");
		if (!ecri)		{
			MessageBox(gblHDlg,"Impossible d'ouvrir le fichier en criture.\nLe nom est probablement faux.",e->nom_fichier,MB_ICONERROR);
			return 0;
		}

	//	FaitNomC(ele_fond[fond]->nom_court);
		ReinitSymTable();
		if ((flags & 3) != PL_OPEN)
			PrepareSymTable(2);
		VerifNomVar(e->nom_court,chneNomVarMap);
		InitSymbole(0,e->nom_base);									//%file%
		InitSymbole(3,sl->nbSprites);								//%size%
		InitSymbole(4,sl->nbSprites*sl->sizeofSD);					//%memory%
		InitSymbole(5,chneNomVarMap);								//%name%
		InitSymbole(6,tilfile->nom_fichier);						//%link%
		InitSymbole(7,&tileX);										//%tilex%
		InitSymbole(8,&tileY);										//%tiley%
		InitSymbole(10,typeSd);										//%type%
		if (userData)
			memcpy(gblUserData, userData, sizeof(gblUserData));
		InitSymbole(15,modele->sdNomModele);						//%model%

		//Ecrit ventuellement ce dont on a besoin dans le ref file
		refFile::writeSectionFiltree(lst_refdefine, e);
		EcritChaineFiltree(lst_declaration,ecri);

		//TRI
		int nBest, bX, bY;
		int *listeX;
		listeSauve.sd=(SPRITE_DEF*)malloc(NB_MAX_ELE_LISTE*sl->sizeofSD);
		listeSauve.sizeofSD=sl->sizeofSD;
		listeX=(int*)malloc(NB_MAX_ELE_LISTE*sizeof(int));
		
		for (i=0;i<sl->nbSprites;i++)		{
			listeX[i]=_SD(*sl,i)->x;
		}

		for (j=0;j<sl->nbSprites;j++)		{
			nBest=-1;
			bX=bY=0x7fffffff;
			for (i=0;i<sl->nbSprites;i++)		{
				if ((_SD(*sl,i)->x<bX) || (_SD(*sl,i)->x==bX && _SD(*sl,i)->y<=bY))		{
	//				if (_SD(sl,i)->y<=bY)		{
						bX=_SD(*sl,i)->x;
						bY=_SD(*sl,i)->y;
						nBest=i;
	//				}
				}
			}

			memcpy(_SD(listeSauve,j),_SD(*sl,nBest), sl->sizeofSD);
			_SD(*sl,nBest)->x=0x7ffffff0;
		}
	//	_SD(sl,nBest)->x=0x7fffffff;

		for (i=0;i<sl->nbSprites;i++)		{
			_SD(*sl,i)->x=listeX[i];
		}

		for (i=0;i<sl->nbSprites;i++)		{
			if (lst_nbChaqueX>0)		{
				if (i%lst_nbChaqueX==0)
					EcritChaineFiltree(lst_chaque_x,ecri);
			}
			if (lst_chaque_valeur_2[0])
			{
				/* Nouveau systme, plus souple */
				gblCurrentSprite = _SD(gblSpriteListe, i);
				EcritChaineFiltree(lst_chaque_valeur_2, ecri);
				gblCurrentSprite = NULL;
			}
			else
				/* Ancien systme */
				analyseChaineTypes(ecri,lst_chaque_valeur_ok,_SD(listeSauve,i));
			
			EcritChaineFiltree(lst_fin_ligne,ecri);
		}
		EcritChaineFiltree(lst_fin_declaration,ecri);

		fclose(ecri);

		free(listeSauve.sd);
		free(listeX);
		return 1;
	}


	int fctEcritBitmapGBA(int flags, int *cmode, ELEMENT *e, ELEMENT *palfile, unsigned long *tableau, unsigned char *palette, int tailleX, int tailleY)			{
		FILE *ecri;
		int i,j, z;

		//Ecrit la bitmap 8 bits
		ecri=fopen(e->nom_fichier,"wb");
		if (!ecri)		{
			CacheTravail();
			MessageBox(gblHDlg,"Impossible d'ouvrir le fichier en criture.\nLe nom est probablement faux ou vous n'avez pas les autorisations d'accs dans ce rpertoire (seriez-vous  l'cole?).",e->nom_fichier,MB_ICONERROR);
			return 0;
		}

		ReinitSymTable();
		PrepareSymTable(2);
		VerifNomVar(e->nom_court,chneNomVarBmp);
		InitSymbole(0,e->nom_base);									//%file%
		InitSymbole(4,tailleX*tailleY);								//%memory%
		InitSymbole(5,chneNomVarBmp);								//%name%
		if (curPF->pfList[*cmode].bUsePalettes)
			InitSymbole(6,palfile->nom_fichier);					//%link%
		else
			InitSymbole(6,"");										//%link%
		InitSymbole(9,curPF->pfList[*cmode].ID);					//%depth%
		if (curPF->pfList[*cmode].ID == 32)
			InitSymbole(10,type32);									//%type%
		else if (curPF->pfList[*cmode].ID == 15)
			InitSymbole(10,type16);									//%type%
		else
			InitSymbole(10,type8);									//%type%

		//Ecrit ventuellement ce dont on a besoin dans le ref file
		refFile::writeSectionFiltree(bmp_refdefine, e);
		EcritChaineFiltree(bmp_declaration,ecri);
	//	sprintf(str,"//%s\n//Converti avec GBA Graphics par Brnni\n//Bitmap 8 bits (\"%s\")\n//Taille: %i*%i\n//Mmoire: %i octets\n\nconst unsigned char %s_bitmap8[%i]=	{\n",els[0].nom_base,els[1].nom_fichier,tailleX,tailleY,tailleX*tailleY,els[1].nom_court,tailleY*tailleX);
	//	fputs(str,ecri);

		for (j=0;j<tailleY;j++)		{
			for (i=0;i<tailleX;i++)		{
				if (*bmp_fin_ligne)
					z=i;
				else
					z=i+j*tailleX;
				if (bmp_nbChaqueX>0 && z>0 && z%bmp_nbChaqueX==0)
					EcritChaineFiltree(bmp_chaque_x,ecri);

				if (curPF->pfList[*cmode].ID == 32)			{
					unsigned char *c = (unsigned char*)&tableau[j*tailleX+i];
					myprintf(ecri,bmp_chaque_valeur_3, RGBA(c[0],c[1],c[2],c[3]));
				}
				else if (curPF->pfList[*cmode].ID == 15)			{
					unsigned char *c = (unsigned char*)&tableau[j*tailleX+i];
					myprintf(ecri,bmp_chaque_valeur_2, Rvba15Bits(c[2],c[1],c[0],!(c[3]&1)));
				}
				else
					myprintf(ecri,bmp_chaque_valeur_1,tableau[j*tailleX+i]);
			}
			EcritChaineFiltree(bmp_fin_ligne,ecri);
		}
		EcritChaineFiltree(bmp_fin_declaration,ecri);

		fclose(ecri);

		return 1;
	}

	void FctGBAApplyMasks()
	{
		if (bGBAPaletteMask)		{
			ptrPFGBA->mf[1].paletteMask = 0xf000;
			ptrPFGBA->mf[1].paletteShift = 12;
		}
		else	{
			ptrPFGBA->mf[1].paletteMask = 0;
			ptrPFGBA->mf[1].paletteShift = 0;
		}
		if (bGBAMirrorMask)		{
			ptrPFGBA->mf[1].hMirrorMask = 0x400;
			ptrPFGBA->mf[1].hMirrorShift = 10;
			ptrPFGBA->mf[1].vMirrorMask = 0x800;
			ptrPFGBA->mf[1].vMirrorShift = 11;
		}
		else	{
			ptrPFGBA->mf[1].hMirrorMask = 0;
			ptrPFGBA->mf[1].hMirrorShift = 0;
			ptrPFGBA->mf[1].vMirrorMask = 0;
			ptrPFGBA->mf[1].vMirrorShift = 0;
			//On augmente le masque
			if (bGBAPaletteMask)
				ptrPFGBA->mf[1].tileMask = 0xfff;
			else
				ptrPFGBA->mf[1].tileMask = 0xffff;
		}
	}

	LRESULT CALLBACK fctPersonnalisationGBAHandler(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)		{
			static HWND hCheckPalette, hCheckMirror, hCheckAlignement, hCheckBGR;

			switch (message)
			{
				case WM_INITDIALOG:
					//Cration des contrles
					hCheckAlignement = MyCreateWindow("Button", "__attribute__((aligned(4)))", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
						10, 5, 160, 18, hDlg);
					hCheckBGR = MyCreateWindow("Button", "Inverser RGB <-> BGR palettes", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
						10, 5+18, 160, 18, hDlg);
					if ((PluginPropertiesWindow::nFlags& 0x7000) == 0x1000)			{
						hCheckMirror = MyCreateWindow("Button", "Bits miroir [10-11]", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
							10, 5+18*2, 150, 18, hDlg);
						hCheckPalette = MyCreateWindow("Button", "Bits palette [12-15]", WS_CHILD | WS_VISIBLE | BS_AUTOCHECKBOX,
							10, 5+18*3, 150, 18, hDlg);
					}
					else
						hCheckMirror = hCheckPalette = NULL;
					SendMessage(hCheckMirror, BM_SETCHECK, bGBAMirrorMask?BST_CHECKED:BST_UNCHECKED, 0);
					SendMessage(hCheckPalette, BM_SETCHECK, bGBAPaletteMask?BST_CHECKED:BST_UNCHECKED, 0);
					SendMessage(hCheckAlignement, BM_SETCHECK, nGBAAlignement ? BST_CHECKED : BST_UNCHECKED, 0);
					SendMessage(hCheckBGR, BM_SETCHECK, bGBAColorOrder ? BST_CHECKED : BST_UNCHECKED, 0);
					break;

				case WM_DESTROY:
					if (hCheckMirror && hCheckPalette)		{
						bGBAMirrorMask = SendMessage(hCheckMirror, BM_GETCHECK, 0, 0);
						bGBAPaletteMask = SendMessage(hCheckPalette, BM_GETCHECK, 0, 0);
					}
					nGBAAlignement = SendMessage(hCheckAlignement, BM_GETCHECK, 0, 0) ? 4 : 0;
					bGBAColorOrder = SendMessage(hCheckBGR, BM_GETCHECK, 0, 0);
					break;
			}
			return FALSE;
	}

	int fctPersonnalisationGBA(int flags, int *cmode)
	{
		int hauteur = 81;
		//Le bouton est actif?
		if (flags & 0x8000)		{
			//(flags & 0x7000) == 0x1000 => mode configurer le plugin, cmode toujours 1
			//(flags & 0x7000) == 0x2000 => depuis l'cran de conversion, peut dpendre du cmode
			//(flags & 0xfff) == SEL_LISTES => uniquement pour les listes, par exemple
			if ((flags & 0x7000) == 0x1000 || (flags & 0x7000) == 0x2000)
				return 1;
			else
				return 0;
		}
		if ((flags & 0x7000) == 0x2000)
			hauteur = 45;
		PluginPropertiesWindow::Create(flags, cmode, SIZE_FEN(170, hauteur), (DLGPROC)fctPersonnalisationGBAHandler);
		FctGBAApplyMasks();
		return 1;
	}

	void FctGBAPaletteMask(char *ptr)		{
		bGBAPaletteMask = GetEntier(ptr);
		FctGBAApplyMasks();
	}

	void FctGBAMirrorMask(char *ptr)		{
		bGBAMirrorMask = GetEntier(ptr);
		FctGBAApplyMasks();
	}

	void FctGBAAlignement(char *ptr)		{
		//En mode GBA uniquement
		if (curPF != ptrPFGBA)
			return;
		nGBAAlignement = GetEntier(ptr);
	}

	void FctGBARevRGB(char *ptr) {
		bGBAColorOrder = GetEntier(ptr);
	}

	IDENTIFICATEUR objGBA[]=	{
		//Commun
		{"palettemask",			FctGBAPaletteMask,	TYPE_FONCTION},
		{"mirrormask",			FctGBAMirrorMask,	TYPE_FONCTION},
		{"align",				FctGBAAlignement,	TYPE_FONCTION},
		{"revcolororder",		FctGBARevRGB,		TYPE_FONCTION},
	};
	INFOS_OBJET infoObjGBA[]={sizeof(objGBA)/sizeof(objGBA[0]), objGBA};

	void fctInitGBA(int flags, PLATFORM_INFO *p)
	{
		//Initialisation
		if (flags == 0)
		{
			IDENTIFICATEUR idNouveau = {"gba", infoObjGBA, TYPE_OBJET};
			ptrPFGBA = p;
			AjouteObjetExterne(&idNouveau);

//			IDENTIFICATEUR_ETENDU idConvTest = {"test", fctTest, TYPE_FONCTION, NULL};
//			AjouteIdConvExterne(&idConvTest);

			bGBAMirrorMask = bGBAPaletteMask = 1;
			bGBAColorOrder = 0;
			nGBAAlignement = 0;
			FctGBAApplyMasks();
		}
		//Switch vers ce plugin
		else if (flags == 1)
		{
		}
		//Switch vers un autre plugin
		else if (flags == 2)
		{
			FctGBAApplyAlign(0);
		}
	}

	void fctSauvegardeGBA(int flags, FILE *f)
	{
		fprintf(f, "  GBA.paletteMask = %s\n", GetBoolText(bGBAPaletteMask));
		fprintf(f, "  GBA.mirrorMask = %s\n", GetBoolText(bGBAMirrorMask));
		fprintf(f, "  GBA.align = %i\n", nGBAAlignement);
		fprintf(f, "  GBA.revColorOrder = %s\n", GetBoolText(bGBAColorOrder));
	}
};
