typedef unsigned char u8;
typedef unsigned short u16;
typedef unsigned long u32;
typedef signed char s8;
typedef signed short s16;
typedef signed long s32;

typedef struct _NODE {
    BOOL bIsLeaf;               // TRUE if node has no children
    UINT nPixelCount;           // Number of pixels represented by this leaf
    UINT nRedSum;               // Sum of red components
    UINT nGreenSum;             // Sum of green components
    UINT nBlueSum;              // Sum of blue components
    struct _NODE* pChild[8];    // Pointers to child nodes
    struct _NODE* pNext;        // Pointer to next reducible node
} NODE;

extern unsigned long *xSourceBitmap;
extern int xSourceWidth, xSourceHeight;
extern HDC xHdcConsole;
extern int xColorsPerPalette, xNbPalettes;
extern int xTileX, xTileY;
//xUnfavorNewColorCreation: -1: dfavorise la cration de nouvelles couleurs (pourri), 0: rien, >0: calcule les diffs entre les couleurs * facteur, -2: calcule pour toute l'image (meilleur rsultat, ultra lent)
//xPostProcessingCompletePalette: post processing qui remplit les palettes inutilises avec les tiles qui le mritent. Utile si unfavorcolorcreation est >0
extern int xUnfavorNewColorCreation, xUseOctree, xSuppressNearColors;
extern int xPostProcessingCompletePalette;

#define xTileMaxX 64
#define xTileMaxY 64
#define xMaxTilesPerPalette 10000
#define xReduceColors 1
#define xMaxColorsPerPalette 256
#define xMaxNbPalettes 256

//u32 xCouleurMasque = RGB(255,0,254);
typedef struct	{
	u32 data[xMaxColorsPerPalette];
	POINT associatedTiles[xMaxTilesPerPalette];
	int nbAssocTiles;
	int usedColors;
} XPALETTE;

extern XPALETTE *xPalettes;
extern int xUsedPalettes;
extern int xGblUsedColors;
extern __int64 xGblScore;

//#define ABS(x) (((x)<0)?(-(x)):((x)))
#define xSrc(x,y) (xSourceBitmap[(xSourceWidth*(y))+(x)])

void xApplyOptimalTilesetPalData(u32 *tableau, u32 *palette, int x, int y, int l, int h, int precisionMax, int nCoul);
void xApplyOptimalTilesetPalDataMultiple(XPALETTE *p, u32 *tableau, u32 *palette, int l, int h, int precisionMax, int nCoul);

inline int xColor32to15(unsigned long c)			{
	return (min((c&0xff)+4,255)>>3)+((min(((c>>8)&0xff)+4,255)>>3)<<5)+((min(((c>>16)&0xff)+4,255)>>3)<<10);
}

inline int xColor15to32(unsigned long c)			{
//	return ((((c&0x1f)<<3)+4)+(((((c>>5)&0x1f)<<3)+4)<<8)+(((((c>>10)&0x1f)<<3)+4)<<16));
	return min(((c&0x1f)<<3)+4,255)+(min((((c>>5)&0x1f)<<3)+4,255)<<8)+(min((((c>>10)&0x1f)<<3)+4,255)<<16);
}

inline int xColor32reduce(unsigned long c)			{
	if (xReduceColors)
		return (min((c&0xff),255)>>3)+((min(((c>>8)&0xff),255)>>3)<<8)+((min(((c>>16)&0xff),255)>>3)<<16);
	else
		return c;
}

inline int xColor32expand(unsigned long c)			{
	if (xReduceColors)
		return min(((c&0x1f)<<3)+4,255)+(min((((c>>8)&0x1f)<<3)+4,255)<<8)+(min((((c>>16)&0x1f)<<3)+4,255)<<16);
	else
		return c;
}

void xDisplayBitmap();
void xDrawAColor(int x, int y, int c);
void xDisplayPalettes();
void xDisplayPalette(u32 *pal);

//Calcule et renvoie le nombre de couleurs et stocke une palette ainsi qu'un tableau
//reprsentant l'image 8 bits en utilisant "nbMax" couleurs et une tolrance de
//"diffrence" units RVB.
int xCalcPalette(u32 *tableau, u8 *palette, int x0, int y0, int l, int h, int difference, int nCoulInitiales, int nbBase, int nbMax);
int xCalcPaletteMultiple(XPALETTE *pal, u32 *tableau, u8 *palette, int l, int h, int difference, int nCoulInitiales, int nbBase, int nbMax);
void test_tablo(int x0, int y0, int l, int h, u32 *tableau, u32 *palette);
void xTestOptimalTilesetOnTile(int palnb, u32 *tableau, u32 *palette, int x, int y, int l, int h, int precisionMax, int nCoul);
void xTestOptimalTilesetOnMultipleTiles(XPALETTE *p, u32 *tableau, u32 *palette, int l, int h, int precisionMax, int nCoul);
int xRenderPalette(u32 *tableau, u8 *palette, int x0, int y0, int l, int h, int difference, int nCoulInitiales, int nbBase, int nbMax);
int xRenderPaletteMultiple(XPALETTE *pal, u32 *tableau, u8 *palette, int l, int h, int difference, int nCoulInitiales, int nbBase, int nbMax);
void xApplyOptimalTilesetPalData(u32 *tableau, u32 *palette, int x, int y, int l, int h, int precisionMax, int nCoul);
void xApplyOptimalTilesetPalDataMultiple(XPALETTE *p, u32 *tableau, u32 *palette, int l, int h, int precisionMax, int nCoul);
void xApplyOptimalTileset(int palnb, u32 *tableau, u32 *palette, int x, int y, int l, int h, int precisionMax, int nCoul);
int xGetBestPaletteForTile(u32 *tableau, u32 *palette, int x, int y, int l, int h, int precisionMax, int nCoul);
void xAddPalette(u32 *palette, int usedColors, int tileX, int tileY);
void xReplacePalette(XPALETTE *p, u32 *palette, int usedColors, int tileX, int tileY);
void xAddTileToPalette(XPALETTE *p, int x, int y);
void xWriteTile(unsigned long **motifs, u32 *tableau);
inline int xColorDiff(int c1, int c2);
__int64 xCalcWholeImageScore();
int xTest(int precisionMax, int nCoul, unsigned long *motifs, unsigned long *map, unsigned long *syspal);
void xInitConversion();
void xDeinitConversion();
unsigned long *xConvertTilesetToBitmap(unsigned long *motifs, int nTiles, int tileX, int tileY);
void xReduceBitmapColorsToPlatform(unsigned long *motifs, int tailleX, int tailleY);
unsigned long *xConvertBitmapToTileset(unsigned long *motifs, int nTiles, int tileX, int tileY);
void xInvertBGR32(unsigned long *motifs, int size);
