/***********************************************
			XREDUCTION
************************************************/

int xNbreCouleursPalette;

BOOL xCreateOctreePalette (unsigned long *tableau, unsigned long *palette, int x0, int y0, int l, int h, UINT nMaxColors, UINT nColorBits);
void xAddColor (NODE**, BYTE, BYTE, BYTE, UINT, UINT, UINT*, NODE**);
NODE* xCreateNode (UINT, UINT, UINT*, NODE**);
void xReduceTree (UINT, UINT*, NODE**);
void xDeleteTree (NODE**);
void xGetPaletteColors (NODE*, PALETTEENTRY*, UINT*);

BOOL xCreateOctreePalette (unsigned long *tableau, unsigned long *palette, int x0, int y0, int l, int h, UINT nMaxColors, UINT nColorBits)
{
    int i, j;
    BYTE r, g, b;
    NODE* pTree;
    UINT nLeafCount, nIndex;
    NODE* pReducibleNodes[9];

    // Initialize octree variables
    pTree = NULL;
    nLeafCount = 0;
    if (nColorBits > 8) // Just in case
        return 0;
    for (i=0; i<=(int) nColorBits; i++)
        pReducibleNodes[i] = NULL;

    for (j=0; j<h; j++) {
	    for (i=0; i<l; i++) {
			b = (u8)((xSrc(i+x0, j+y0)>>16)&0xff);
			g = (u8)((xSrc(i+x0, j+y0)>>8)&0xff);
			r = (u8)(xSrc(i+x0, j+y0)&0xff);
			//Vide
			if (!(xSrc(i+x0, j+y0) & 0xff000000))		{
//				tableau[l*y+x]=coulTransparente;
				continue;
			}
			xAddColor (&pTree, r, g, b, nColorBits, 0, &nLeafCount,
					  pReducibleNodes);
			while (nLeafCount > nMaxColors)
				xReduceTree (nColorBits, &nLeafCount, pReducibleNodes);
        }
    }

    if (nLeafCount > nMaxColors) { // Sanity check
        xDeleteTree (&pTree);
        return 0;
    }

    nIndex = 0;
	if (!pTree)
		return 0;
    xGetPaletteColors (pTree, (PALETTEENTRY*)palette, &nIndex);
	xNbreCouleursPalette=nLeafCount;
    xDeleteTree (&pTree);
    return 1;
}

BOOL xCreateOctreePaletteMultiple (XPALETTE *pal, unsigned long *tableau, unsigned long *palette, int l, int h, UINT nMaxColors, UINT nColorBits)
{
    int i, j, x0, y0, p;
    BYTE r, g, b;
    NODE* pTree;
    UINT nLeafCount, nIndex;
    NODE* pReducibleNodes[9];

    // Initialize octree variables
    pTree = NULL;
    nLeafCount = 0;
    if (nColorBits > 8) // Just in case
        return 0;
    for (i=0; i<=(int) nColorBits; i++)
        pReducibleNodes[i] = NULL;

	for (p=0;p<pal->nbAssocTiles;p++)		{
		x0 = pal->associatedTiles[p].x;
		y0 = pal->associatedTiles[p].y;
		for (j=0; j<h; j++) {
			for (i=0; i<l; i++) {
				b = (u8)((xSrc(i+x0, j+y0)>>16)&0xff);
				g = (u8)((xSrc(i+x0, j+y0)>>8)&0xff);
				r = (u8)(xSrc(i+x0, j+y0)&0xff);
				if (!(xSrc(i+x0, j+y0) & 0xff000000))		{
	//				tableau[l*y+x]=coulTransparente;
					continue;
				}
				xAddColor (&pTree, r, g, b, nColorBits, 0, &nLeafCount,
						  pReducibleNodes);
				while (nLeafCount > nMaxColors)
					xReduceTree (nColorBits, &nLeafCount, pReducibleNodes);
			}
		}
	}

    if (nLeafCount > nMaxColors) { // Sanity check
        xDeleteTree (&pTree);
        return 0;
    }

	if (!pTree)
		return 0;
    nIndex = 0;
    xGetPaletteColors (pTree, (PALETTEENTRY*)palette, &nIndex);
	xNbreCouleursPalette=nLeafCount;
    xDeleteTree (&pTree);
    return 1;
}


void xAddColor (NODE** ppNode, BYTE r, BYTE g, BYTE b, UINT nColorBits,
    UINT nLevel, UINT* pLeafCount, NODE** pReducibleNodes)
{
    int nIndex, shift;
    static BYTE mask[8] = { 0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01 };

    // If the node doesn't exist, create it
    if (*ppNode == NULL)
        *ppNode = xCreateNode (nLevel, nColorBits, pLeafCount,
                              pReducibleNodes);

    // Update color information if it's a leaf node
    if ((*ppNode)->bIsLeaf) {
        (*ppNode)->nPixelCount++;
        (*ppNode)->nRedSum += r;
        (*ppNode)->nGreenSum += g;
        (*ppNode)->nBlueSum += b;
    }

    // Recurse a level deeper if the node is not a leaf
    else {
        shift = 7 - nLevel;
        nIndex = (((r & mask[nLevel]) >> shift) << 2) |
            (((g & mask[nLevel]) >> shift) << 1) |
            ((b & mask[nLevel]) >> shift);
        xAddColor (&((*ppNode)->pChild[nIndex]), r, g, b, nColorBits,
                  nLevel + 1, pLeafCount, pReducibleNodes);
    }
}

NODE* xCreateNode (UINT nLevel, UINT nColorBits, UINT* pLeafCount,
                  NODE** pReducibleNodes)
{
    NODE* pNode;

    if ((pNode = (NODE*) HeapAlloc (GetProcessHeap (), HEAP_ZERO_MEMORY,
        sizeof (NODE))) == NULL)
        return NULL;

    pNode->bIsLeaf = (nLevel == nColorBits) ? TRUE : FALSE;
    if (pNode->bIsLeaf)
        (*pLeafCount)++;
    else { // Add the node to the reducible list for this level
        pNode->pNext = pReducibleNodes[nLevel];
        pReducibleNodes[nLevel] = pNode;
    }
    return pNode;
}

void xReduceTree (UINT nColorBits, UINT* pLeafCount, NODE** pReducibleNodes)
{
    int i;
    NODE* pNode;
    UINT nRedSum, nGreenSum, nBlueSum, nChildren;

    // Find the deepest level containing at least one reducible node
    for (i=nColorBits - 1; (i>0) && (pReducibleNodes[i] == NULL); i--);

    // Reduce the node most recently added to the list at level i
    pNode = pReducibleNodes[i];
    pReducibleNodes[i] = pNode->pNext;

    nRedSum = nGreenSum = nBlueSum = nChildren = 0;
    for (i=0; i<8; i++) {
        if (pNode->pChild[i] != NULL) {
            nRedSum += pNode->pChild[i]->nRedSum;
            nGreenSum += pNode->pChild[i]->nGreenSum;
            nBlueSum += pNode->pChild[i]->nBlueSum;
            pNode->nPixelCount += pNode->pChild[i]->nPixelCount;
            HeapFree (GetProcessHeap (), 0, pNode->pChild[i]);
            pNode->pChild[i] = NULL;
            nChildren++;
        }
    }

    pNode->bIsLeaf = TRUE;
    pNode->nRedSum = nRedSum;
    pNode->nGreenSum = nGreenSum;
    pNode->nBlueSum = nBlueSum;
    *pLeafCount -= (nChildren - 1);
}

void xDeleteTree (NODE** ppNode)
{
    int i;

    for (i=0; i<8; i++) {
        if ((*ppNode)->pChild[i] != NULL)
            xDeleteTree (&((*ppNode)->pChild[i]));
    }
    HeapFree (GetProcessHeap (), 0, *ppNode);
    *ppNode = NULL;
}

void xGetPaletteColors (NODE* pTree, PALETTEENTRY* pPalEntries, UINT* pIndex)
{
    int i;

    if (pTree->bIsLeaf) {
        pPalEntries[*pIndex].peRed =
            (BYTE) ((pTree->nRedSum) / (pTree->nPixelCount));
        pPalEntries[*pIndex].peGreen =
            (BYTE) ((pTree->nGreenSum) / (pTree->nPixelCount));
        pPalEntries[*pIndex].peBlue =
            (BYTE) ((pTree->nBlueSum) / (pTree->nPixelCount));
		//L'alpha est TOUJOURS  1, octree ne supporte pas la transparence!!!!
        pPalEntries[*pIndex].peFlags = 0xff;
        (*pIndex)++;
    }
    else {
        for (i=0; i<8; i++) {
            if (pTree->pChild[i] != NULL)
                xGetPaletteColors (pTree->pChild[i], pPalEntries, pIndex);
        }
    }
}

void xDisplayPalette(u32*);
int xReductionOctree(unsigned long *tableau, unsigned long *palette, int x0, int y0, int l, int h, int nCoulInitiales, int nCoul)		{
	if (nCoul-nCoulInitiales<=1)
		return 0;

	xNbreCouleursPalette=0;
	xCreateOctreePalette(tableau,palette+nCoulInitiales,x0,y0,l,h,nCoul-nCoulInitiales,8);

	if (xNbreCouleursPalette<=1)
		return 0;

	xApplyOptimalTilesetPalData(tableau, palette, x0, y0, l, h, 250000, xNbreCouleursPalette);

	return xNbreCouleursPalette+nCoulInitiales;
}

int xReductionOctreeMultiple(XPALETTE *p, unsigned long *tableau, unsigned char *palette, int l, int h, int nCoulInitiales, int nCoul)		{
	if (nCoul-nCoulInitiales<=1)
		return 0;

	xNbreCouleursPalette=0;
	xCreateOctreePaletteMultiple(p,tableau,(u32*)palette+nCoulInitiales,l,h,nCoul-nCoulInitiales,8);

	if (xNbreCouleursPalette<=1)
		return 0;

	xApplyOptimalTilesetPalDataMultiple(p, tableau, (u32*)palette, l, h, 250000, xNbreCouleursPalette);

	return xNbreCouleursPalette+nCoulInitiales;
}

